#! perl -w

# Licensed Materials - Property of IBM                       
# Component IBM Rational ROSE_RT BUILD : TARGET_RTS                 
# Copyright IBM Corporation 1999, 2008. All Rights Reserved.      
# U.S. Government Users Restricted Rights -  Use, duplication or    
# disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.



# rtcomp.pl - Compilation controller

# Unbuffer stdout

select STDERR; $| = 1;
select STDOUT; $| = 1;

# load in the error handler

$codegen = $0;
$codegen =~ s{\\}{/}g;
$codegen =~ s{/rtcomp.pl$}{};

require "$codegen/rterror.pl";

# Silence perl: other uses are in rterror.pl.
$rterror::maxError = $rterror::maxError;

# Get the arguments

$component = '';
$depfile   = '';
@deppath   = ();
$depvar    = '';
$spacify   = '';
$srcfile   = '';
$srcobject = '';
$vendor    = 'generic';

while( $#ARGV >= 0 )
{
    $arg = shift @ARGV;
    if   ( $arg eq '-vendor'    ) { $vendor      = shift @ARGV; }
    elsif( $arg eq '-spacify'   ) { $spacify     = shift @ARGV; }
    elsif( $arg eq '-component' ) { $component   = shift @ARGV; }
    elsif( $arg eq '-dep'       ) { $depvar      = shift @ARGV;
				    $depfile     = shift @ARGV; }
    elsif( $arg eq '-I'         ) { push @deppath, shift @ARGV; }
    elsif( $arg eq '-src'       ) { $srcobject   = shift @ARGV;
				    $srcfile     = shift @ARGV; }
    elsif( $arg eq '--'         ) { last; }
    else
    {
	&rterror::action_message( "Unknown rtcomp.pl option $arg", 1 );
    }
}

unless( $srcfile )
{
    &rterror::action_message( 'rtcomp.pl: missing -src option', 1 );
    exit 1;
}

foreach $loc ( @deppath )
{
    $loc =~ s{\\}{/}g;
}

# pick out the src directory

$srcdir = $srcfile;
$srcdir =~ s{(.*)[/\\][^/\\]*}{$1};

# load in configs (for now, just the compiler)

require "$codegen/compiler/$vendor/comp.pl";

print STDERR '!> Compiling ', $srcobject, "\n";

foreach $arg ( @ARGV )
{
    $arg =~ s{^.*\s.*$}{"$&"};
}
$command = join( ' ', @ARGV );

$OS          = '?' unless $OS          = $ENV{'OS'};
$ROSERT_HOST = '?' unless $ROSERT_HOST = $ENV{'ROSERT_HOST'};

$errRedirect = ( $ROSERT_HOST eq 'win32' && $OS !~ m{^Windows_NT$}i )
			? ''
			: '2>&1';

open ERRORS, "$command $errRedirect |";

@errors = <ERRORS>;

close ERRORS;
$retcode = $?;

print @errors;
&rterror::handle_errors();

####
#### MAKEDEPEND
####

sub find_file($)
{
    my( $file ) = @_;
    my( $path );

    # a special case: has a complete path already been provided?
    return $file if -e "$file";

    foreach $loc ( @deppath )
    {
	$path = "$loc/$file";
	return $path if -e $path;
    }

    return '';
}

if( $spacify ne 'none' )
{
    # srcfile is first, let's make all the slashes go one way
    $srcfile =~ s{\\}{/}g;
    @deplist = ( $srcfile );
    $deps{$srcfile} = 1;

    foreach $file ( @deplist )
    {
	unless( open FILE, $file )
	{
	    &rterror::action_message( "Could not open [$file]", 1 );
	    exit 1;
	}

	while( <FILE> )
	{
	    # recognize: include <X> or include "X"
	    if( m{^\s*#\s*include\s*\<([^\>]*)\>} or
		m{^\s*#\s*include\s*\"([^\"]*)\"} )
	    {
		my( $depfile ) = &find_file( $1 );

		if( $depfile && ! $deps{$depfile} )
		{
		    $deps{$depfile} = 1;
		    push @deplist, $depfile;
		}
	    }
	}

	close FILE;
    }

    if( $spacify eq 'dq' )
    {
	foreach $dep ( @deplist )
	{
	    $dep =~ s{^.*\s.*$}{"$&"};
	}
    }
    elsif( $spacify eq 'bs' )
    {
	foreach $dep ( @deplist )
	{
	    $dep =~ s{\s}{\\$&}g;
	}
    }

    $depfile = "$srcobject.dep"    unless $depfile;
    $depvar  = "${srcobject}_DEPS" unless $depvar;

    unless( open DEPFILE, ">$depfile" )
    {
	&rterror::action_message( "Cannot write to '$depfile'", 0 );
	exit 1;
    }

    binmode( DEPFILE );

    print DEPFILE $depvar, " =";

    foreach $dep ( @deplist )
    {
	print DEPFILE " \\\n\t", $dep;
    }

    print DEPFILE "\n";

    close DEPFILE;
}

if( $rterror::worstError < $rterror::maxError )
{
    if( $rterror::worstError == 0 )
    {
	exit 1;
    }
    if( (lc($vendor) eq "diab") || (lc($vendor) eq "gnu") )
    {
        if($retcode & 0xff)
        {
            $mycode = $retcode & 0xff;
            if($mycode == 1)
            { 	
                &rterror::action_message( " caught signal [$retcode] from [$command]", 0);
                exit 1;
            }
        }
        elsif( $retcode )
        { 
            $retcode >>= 8;
            if($retcode == 1) 
            {
                &rterror::action_message( " caught signal [$retcode] from [$command]", 0);
                exit 1;
            }
        }
    }
}
elsif( $retcode & 0xff )
{
    &rterror::action_message( "caught signal [$retcode] from [$command]", 0 );
    exit 1;
}
elsif( $retcode )
{
    $retcode >>= 8;
    &rterror::action_message( "detected error [$retcode] from [$command]", 0 );
    exit 1;
}
