/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTMain_h__
#define __RTMain_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

class  RTActor;
class  RTActorRef;
class  RTController;
class  RTSuperActor;
struct RTSystemDescriptor;

class RTMain
{
private:
    friend class RTSuperActor;

    static int                          execute( void );
    static int                          mainLine( void );

    static void                         targetStartup( void );
    static void                         installHandlers( void );
    static void                         installOneHandler( int,
							   void (*)( void ) );
    static void                         removeHandlers( void );
    static void                         removeOneHandler( int );
    static void                         targetShutdown( void );

    static const RTSystemDescriptor   * _system;
    static int                          _argc;
    static const char * const         * _argv;

    static RTController               * _controller;
    static RTActor                    * _superActor;
    static RTActorRef                 * _topReference;

#if USE_THREADS && OTRTSDEBUG != DEBUG_NONE
    static int                          debugPriority;
    static int                          debugStackSize;
#endif

public:
    // entryPoint must be provided by the generated code
    static int                          entryPoint( int, const char * const * );

    static int                          shutdown( RTController * );
    static const char                 * debugName( void );

    static RTS_INLINE RTController    * controller( void );
    static RTS_INLINE RTActorRef      * topReference( void );
    static RTS_INLINE RTActor         * topActor( void );

    static RTS_INLINE
	   const RTSystemDescriptor      * systemInfo( void );
    static RTS_INLINE int                  argCount( void );
    static RTS_INLINE const char * const * argStrings( void );

#if USE_THREADS
    static void                         lock( void );
    static void                         unlock( void );
#else
    static RTS_INLINE void              lock( void );
    static RTS_INLINE void              unlock( void );
#endif
};

#if RTS_INLINES
#include <RTMain.inl>
#endif

#endif // __RTMain_h__
