/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTSymmetricSignal_inl__
#define __RTSymmetricSignal_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTObject_class_h__
#include <RTObject_class.h>
#endif

#ifndef __RTSymmetricSignal_h__
#include <RTSymmetricSignal.h>
#endif

RTS_INLINE RTSymmetricSignal::RTSymmetricSignal( RTProtocol * port,
						 int          in_signal,
						 int          out_signal )
    : _port      ( port       )
    , _in_signal ( in_signal  )
    , _out_signal( out_signal )
{
}

RTS_INLINE int RTSymmetricSignal::purge( void )
{
    return _port->purge( _in_signal );
}

RTS_INLINE int RTSymmetricSignal::purgeAt( int index )
{
    return _port->purgeAt( _in_signal, index );
}

RTS_INLINE int RTSymmetricSignal::recall( int front )
{
    return _port->recall( _in_signal, front );
}

RTS_INLINE int RTSymmetricSignal::recallAt( int index, int front )
{
    return _port->recallAt( _in_signal, index, front );
}

RTS_INLINE int RTSymmetricSignal::recallAll( int front )
{
    return _port->recallAll( _in_signal, front );
}

RTS_INLINE int RTSymmetricSignal::recallAllAt( int index, int front )
{
    return _port->recallAllAt( _in_signal, index, front );
}

RTS_INLINE int RTSymmetricSignal::invoke( RTMessage * replyBuffers )
{
    return _port->invoke( replyBuffers, _out_signal, (const void *)0,
						     &RTType_void );
}

RTS_INLINE int RTSymmetricSignal::invokeAt( int index, RTMessage * replyBuffer )
{
    return _port->invokeAt( index, replyBuffer, _out_signal, (const void *)0,
							     &RTType_void );
}

RTS_INLINE int RTSymmetricSignal::reply( void )
{
    return _port->reply( _out_signal, (const void *)0, &RTType_void );
}

RTS_INLINE int RTSymmetricSignal::send( int priority )
{
    return _port->send( _out_signal, (const void *)0, &RTType_void, priority );
}

RTS_INLINE int RTSymmetricSignal::sendAt( int index, int priority )
{
    return _port->sendAt( index, _out_signal, (const void *)0,
					      &RTType_void,
					      priority );
}

#endif // __RTSymmetricSignal_inl__
