/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTTime_h__
#define __RTTime_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTFieldDescriptor_h__
#include <RTFieldDescriptor.h>
#endif

#ifndef __RTInteger_h__
#include <RTInteger.h>
#endif

#ifndef __RTSequence_h__
#include <RTSequence.h>
#endif

struct RTTimespec;

class RTTime : public RTSequence
{
public:
    RTInteger               hours;
    RTInteger               minutes;
    RTInteger               seconds;

    explicit                RTTime( int _hrs = 0, int _min = 0, int _sec = 0 );
			    RTTime( const RTTime & );
			    RTTime( const RTTimespec & ); //lint !e1931
    virtual                 ~RTTime( void );

    static RTTime           now( void ); /* current time */

    int                     asInt( void ) const; /* seconds from midnight */

    /* return time based upon number of seconds from midnight */
    static RTTime           asTime( int value );

    RTTime                & operator= ( const RTTime & );
    RTTime                & operator+=( const RTTime & );
    RTTime                & operator-=( const RTTime & );

    virtual int             operator==( const RTDataObject & ) const;

    virtual const RTObject_class   * getClassData( void ) const;
    static  const RTObject_class     classData;
    static  const RTFieldDescriptor  fieldData[];

#define RTType_RTTime ( RTTime::classData )
};

int            operator< ( const RTTime &, const RTTime & );
RTS_INLINE int operator> ( const RTTime &, const RTTime & );
RTS_INLINE int operator<=( const RTTime &, const RTTime & );
RTS_INLINE int operator>=( const RTTime &, const RTTime & );
RTTime         operator+ ( const RTTime &, const RTTime & );
RTTime         operator- ( const RTTime &, const RTTime & );

struct RTTypedValue_RTTime
{
    const void           * data;
    const RTObject_class * type;

    RTS_INLINE RTTypedValue_RTTime( const RTTime         & ); //lint !e1931
    RTS_INLINE RTTypedValue_RTTime( const RTTime         &,
				    const RTObject_class * );
}; //lint !e1712

#if RTS_INLINES
#include <RTTime.inl>
#endif

#endif // __RTTime_h__
