/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTFormat.h>

static const char * const priority_name[ OTRTS_NUMPRIO ] =
{
    "Synchronous",
    "System     ",
    "Panic      ",
    "High       ",
    "General    ",
    "Low        ",
    "Background "
};

static void printValue( RTDiagStream & out, unsigned value )
{
    static const char space[] = "           ";
    char              buffer[ 40 ];
    int               size = RTFormat::_unsigned( (unsigned long)value,
						  buffer,
						  (int)sizeof( buffer ) );

    out << &space[ size ] << buffer;
}

void RTController::printStats( void ) const
{
    RTDiagStream & out      = RTDiag::out();
    int            priority;

    out << "\n" << name() << "\n  " << strerror() << "\n";

#if RTS_COUNT
    out << "\n  actors incarnated : " << counts.actorsIncarnated
	<< "\n  actors destroyed  : " << counts.actorsDestroyed
	<< "\n  ports incarnated  : " << counts.portsIncarnated
	<< "\n  ports destroyed   : " << counts.portsDestroyed
	<< "\n"
	<< "\n  Messages allocated      : " << counts.messagesAllocated
	<< "\n  Peak message allocation : " << counts.peakMessagesAllocated
	<< "\n";

    unsigned totalDelivered = 0U;
#endif
    unsigned totalInternal  = 0U;
#if USE_THREADS
    unsigned totalIncoming  = 0U;
#endif

    out << "\n  messages     ";
    out << "     queued";
#if USE_THREADS
    out << "   incoming";
#endif
#if RTS_COUNT
    out << "  delivered";
#endif

    for( priority = 0; priority < OTRTS_NUMPRIO; ++priority )
    {
	unsigned value;

	out << "\n    " << priority_name[ priority ];
	value = (unsigned)internalQ[ priority ].size();
	printValue( out, value );
	totalInternal += value;
#if USE_THREADS
	value = (unsigned)incomingQ[ priority ].size();
	printValue( out, value );
	totalIncoming += value;
#endif
#if RTS_COUNT
	value = counts.messagesDelivered[ priority ];
	printValue( out, value );
	totalDelivered += value;
#endif
    }

    out << "\n    Total      ";
	printValue( out, totalInternal );
#if USE_THREADS
	printValue( out, totalIncoming );
#endif
#if RTS_COUNT
	printValue( out, totalDelivered );
#endif
    out << "\n";
}
