/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTMemoryUtil.h>
#include <RTObject_class.h>

static void string_init( const RTObject_class *, void * target )
{
    *(char **)target = (char *)0;
}

static void string_copy( const RTObject_class *,
			 void                 * target,
			 const void           * source )
{
    *(char **)target = RTMemoryUtil::strdup( *(const char * const *)source );
}

#if OBJECT_DECODE
static int string_decode( const RTObject_class *,
			  void                 * target,
			  RTDecoding           * coding )
{
    char * string;

    if( coding->get_string( string ) == 0 )
	return 0;

    delete [] *(char **)target;

    *(char **)target = string;

    return 1;
}
#endif

#if OBJECT_ENCODE
static int string_encode( const RTObject_class *,
			  const void           * source,
			  RTEncoding           * coding )
{
    return coding->put_string( *(char * const *)source );
}
#endif

static void string_destroy( const RTObject_class *, void * target )
{
    char ** value = (char **)target;

    delete [] *value;

    *value = (char *)0;
}

const RTObject_class RTType_RTpchar =
{
    (RTObject_class *)0,
    (RTSuperAccessFunction)0,
    "RTpchar",
    (RTVersionId)0,
    (RTFieldOffset)sizeof( char * ),
    string_init,
    string_copy,
#if OBJECT_DECODE
    string_decode,
#endif
#if OBJECT_ENCODE
    string_encode,
#endif
    string_destroy,
    0, (const RTFieldDescriptor *)0
};

#if OBJECT_DECODE
RTTypeInstaller RTpchar_installer( RTType_RTpchar );
#endif
