/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2010. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTLayerConnector.h>
#include <RTMemoryUtil.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>

int RTProtocol::registerSAP( const char * name, CallStyle style )
{
    RTController        * controller = context();
    RTController::Error   error;

    if( ( _info->properties & RTPortDescriptor::MaskKind )
			   != RTPortDescriptor::KindUnwired ||
	( style == Programmatic &&
	( _info->properties & RTPortDescriptor::MaskRegisterLocked ) != 0 ) )
    {
	error = RTController::badOperation;
    }
    else if( name == (const char *)0 )
    {
	error = RTController::badValue;
    }
    else for(;;) // guess: all will go smoothly
    {
	if( isRegistered() != 0 )
	{
	    if( ( getFlags() & RegisteredAsSAP ) != 0 &&
		RTMemoryUtil::strcmp( name, getRegisteredName() ) == 0 )
	    {
		return 1;
	    }

	    (void)deregister();
	}

	controller->trace( RTController::LayerRegisterSAP, this, name );

	RTLayerConnector::lock();

	_layer = RTLayerConnector::registerSAP( controller, this, name );

	if( _layer != (RTLayerData *)0 )
	{
	    setFlag( RegisteredAsSAP );
	    RTLayerConnector::unlock();
	    return 1;
	}

	RTLayerConnector::unlock();
	error = RTController::reg;
	break;
    } // endguess

    controller->setError( error, RTController::LayerRegisterSAP, this, name );

    return 0;
}
