/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTSoleController.h>
#include <RTSpecialPorts.h>
#include <RTSpecialSignals.h>
#include <RTTimerActor.h>

#include <errno.h>
#include <signal.h>

void RTSoleController::waitForEvents( void )
{
    RTActor * timerAgent;

    if( timer_interrupts != 0 )
    {
	timer_interrupts = 0;
	if( (timerAgent = RTTimerActor::soleInstance()) != (RTActor *)0 )
	    timerAgent->rtsBehavior( RTInterruptSignal, RTControlPort );
	return;
    }

    sigset_t mask_all;
    sigset_t mask_none;
    sigset_t mask_old;

    // mask all signals before checking signalCounts
    if( sigfillset( &mask_all ) != 0 )
	RTDiag::panic( "sigfillset" );
    if( sigprocmask( SIG_BLOCK, &mask_all, &mask_old ) != 0 )
	RTDiag::panic( "sigprocmask" );

    // if no signals occurred since the last call to sigsuspend we will wait
    if( timer_interrupts == 0 ) //lint !e774
    {
	// wait for SIGALRM which might trigger a timeout
	if( sigemptyset( &mask_none ) != 0 )
	    RTDiag::panic( "sigemptyset" );
	if( ( sigsuspend( &mask_none ) == -1 ) && ( errno != EINTR ) )
	    RTDiag::panic( "sigsuspend" );
    }

    // restore the signal mask
    if( sigprocmask( SIG_SETMASK, &mask_old, (sigset_t *)0 ) != 0 )
	RTDiag::panic( "sigprocmask" );
}
