/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTSpecialSignals.h>
#include <RTTimerActor.h>
#include <RTTimerNode.h>
#include <RTTimespec.h>

void RTTimerActor::informAgain( RTTimerNode * node )
{
#if USE_THREADS
    RTTimerNode::critical.enter();
#endif

    RTTimespec now;

    getRelativeClock( now );

    RTTimespec delta( now );
    RTTimespec interval( node->repeatInterval );

    delta -= node->expireTime;

    RTAssert( delta    >= RTTimespec::zero ); //lint !e909
    RTAssert( interval >  RTTimespec::zero ); //lint !e909

    int log_scale = 0;
    int scale     = 1;

    while( interval < delta ) //lint !e909
    {
	log_scale += 1;
	scale     *= 2;
	interval  += interval;
    }

    node->missedIntervals = 0;

    for( ; log_scale > 0; --log_scale, scale /= 2 )
    {
	node->expireTime += interval;
	if( node->expireTime > now ) //lint !e909
	    node->expireTime -= interval;
	else
	    node->missedIntervals += scale;

	interval.tv_nsec /= 2L;
	if( ( interval.tv_sec & 1L ) != 0L )
	    interval.tv_nsec += 500000000L;
	interval.tv_sec /= 2L;
    }

    if( node->expireTime <= now ) //lint !e909
    {
	node->expireTime      += node->repeatInterval;
	node->missedIntervals += 1;
    }

    if( scale <= 0 ) // overflow?
	// limit to maximum int value
	node->missedIntervals = ( ~ 0U ) >> 1;

    int atHead = pendingRelative.insert( node );

#if USE_THREADS
    RTTimerNode::critical.leave();

    if( atHead != 0 )
	(void)node->owner->send( this, RTWakeupSignal,
				       (const void *)0,
				       (const RTObject_class *)0, System );
#else
    (void)atHead;
#endif
}
