/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTDiag.h>
#include <RTMessage.h>
#include <RTTimerActor.h>
#include <RTTimerId.h>
#include <RTTimerNode.h>
#include <RTTiming.h>

RTMessage::DeliverResult RTTimerNode::_deliver( void )
{
    // cancel must not delete this (repeating) timeout
    active = 1;

    RTMessage    * thisMsg  = msg;
    RTMessage    * nextMsg  = (RTMessage *)0;
    RTTimerActor * timerActor;
    RTActor      * receiver = thisMsg->receiver();

    if( ( repeatInterval.tv_sec | repeatInterval.tv_nsec ) == 0L )
    {
	// invalidate all RTTimerId objects
	// that refer to this (one-shot) timeout
	_disconnectAll();
	timerActor = (RTTimerActor *)0;
    }
    else if( (timerActor = RTTimerActor::soleInstance()) == (RTTimerActor *)0 )
    {
	owner->setError( RTController::noConnect,
			 RTController::TimerInform,
			 sap,
			 (const void *)0 );
    }
    else if( (nextMsg = owner->newMsg()) == (RTMessage *)0 )
    {
	owner->setError( RTController::noMem,
			 RTController::SystemService,
			 (const void *)0,
			 (const void *)0 );
    }
    else
    {
	buildMsg( nextMsg, receiver, thisMsg->getPriority() );
    }

    msg = nextMsg;

#if USE_THREADS
    critical.leave();
#endif

    if( nextMsg != (RTMessage *)0 )
	timerActor->informAgain( this );

    thisMsg->clear();
    thisMsg->setSignal( Timing::Base::rti_timeout );
    thisMsg->setData( data, type );
    thisMsg->setDestination( receiver, sap, 0 );

    RTMessage::DeliverResult result = thisMsg->deliver();

#if USE_THREADS
    critical.enter();
#endif

    active = 0;

    if( msg == (RTMessage *)0 )
	owner->freeTimerNode( this );

    return result;
}

RTMessage::DeliverResult RTTimerNode::deliver( const RTMessage * message )
{
    RTMessage::DeliverResult result;

    RTAssert( message->getType() == &RTType_RTTimerId );

#if USE_THREADS
    critical.enter();
#endif

    RTTimerId   * id   = (RTTimerId *)message->data;
    RTTimerNode * node = id->node;

    if( node == (RTTimerNode *)0 ) // cancelled
	result = RTMessage::Accepted;
    else
	result = node->_deliver();

#if USE_THREADS
    critical.leave();
#endif

    return result;
}
