/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTPriority.h>
#include <endeBoth.h>
#include <endePriv.h>
#include <endeTarg.h>

/*||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||
||  Stubs, since we shouldn't link in the real toolset functions here
||
+|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/

extern "C"
{

int decResultCo( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtTargI( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtPrese( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtRefer( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtChain( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtVaria( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtMsgOu( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtMsgIn( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtMsgOI( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtRtsSt( TO_bufferInfo *, TO_structInfo * ) { return 0; }
int decEvtHisto( TO_bufferInfo *, TO_structInfo * ) { return 0; }

void freeEvtTargI( TO_structInfo * ) {}
void freeEvtPrese( TO_structInfo * ) {}
void freeEvtRefer( TO_structInfo * ) {}
void freeEvtChain( TO_structInfo * ) {}
void freeEvtVaria( TO_structInfo * ) {}
void freeEvtMsgOu( TO_structInfo * ) {}
void freeEvtMsgIn( TO_structInfo * ) {}
void freeEvtMsgOI( TO_structInfo * ) {}
void freeEvtHisto( TO_structInfo * ) {}

/*----------------------------------------------------------------------------
|  Decoding (service routines)
+---------------------------------------------------------------------------*/

static int decodeDaemon( TO_bufferInfo * buf, int func, TO_structInfo * info )
{
    TO_daemonSett * res = (TO_daemonSett *)initStructInfo( info, func );

    if( buf == (TO_bufferInfo *)0 || res == (TO_daemonSett *)0 )
	return DEC_BAD_PARM;

    if( doDecodeStr( buf, &res->settings ) != DEC_SUCCESS )
    {
	TO_freeStructInfo( info );
	return buf->status;
    }

    return buf->status = DEC_SUCCESS;
}

static int decodeActor( TO_bufferInfo * buf, int func, TO_structInfo * info )
{
    TO_actorInfo * res = (TO_actorInfo *)initStructInfo( info, func );

    if( buf == (TO_bufferInfo *)0 || res == (TO_actorInfo *)0 )
	return DEC_BAD_PARM;

    if( doDecodeStr( buf, &res->actor     ) != DEC_SUCCESS ||
	doDecodeInt( buf, &res->allRecurs ) != DEC_SUCCESS )
    {
	TO_freeStructInfo( info );
	return buf->status;
    }

    return buf->status = DEC_SUCCESS;
}

static int decodePort( TO_bufferInfo * buf, int func, TO_structInfo * info )
{
    TO_portInfo * res = (TO_portInfo *)initStructInfo( info, func );

    if( buf == (TO_bufferInfo *)0 || res == (TO_portInfo *)0 )
	return DEC_BAD_PARM;

    if( doDecodeStr( buf, &res->actor ) != DEC_SUCCESS ||
	doDecodeStr( buf, &res->port  ) != DEC_SUCCESS )
    {
	TO_freeStructInfo( info );
	return buf->status;
    }

    return buf->status = DEC_SUCCESS;
}

static int decodeState( TO_bufferInfo * buf, int func, TO_structInfo * info )
{
    TO_stateInfo * res = (TO_stateInfo *)initStructInfo( info, func );

    if( buf == (TO_bufferInfo *)0 || res == (TO_stateInfo *)0 )
	return DEC_BAD_PARM;

    if( doDecodeStr( buf, &res->actor ) != DEC_SUCCESS ||
	doDecodeStr( buf, &res->state ) != DEC_SUCCESS )
    {
	TO_freeStructInfo( info );
	return buf->status;
    }

    return buf->status = DEC_SUCCESS;
}

static int decodeChain( TO_bufferInfo * buf, int func, TO_structInfo * info )
{
    TO_chainInfo * res = (TO_chainInfo *)initStructInfo( info, func );

    if( buf == (TO_bufferInfo *)0 || res == (TO_chainInfo *)0 )
	return DEC_BAD_PARM;

    if( doDecodeStr( buf, &res->actor    ) != DEC_SUCCESS ||
	doDecodeStr( buf, &res->state    ) != DEC_SUCCESS ||
	doDecodeStr( buf, &res->chain    ) != DEC_SUCCESS ||
	doDecodeInt( buf, &res->position ) != DEC_SUCCESS )
    {
	TO_freeStructInfo( info );
	return buf->status;
    }

    return buf->status = DEC_SUCCESS;
}

static int decodeAddDelVar( TO_bufferInfo * buf,
			    int             func,
			    TO_structInfo * info )
{
    TO_variableInfo * res = (TO_variableInfo *)initStructInfo( info, func );

    if( buf == (TO_bufferInfo *)0 || res == (TO_variableInfo *)0 )
	return DEC_BAD_PARM;

    if( doDecodeStr( buf, &res->actor    ) != DEC_SUCCESS ||
	doDecodeStr( buf, &res->variable ) != DEC_SUCCESS )
    {
	TO_freeStructInfo( info );
	return buf->status;
    }

    return buf->status = DEC_SUCCESS;
}

static int decodeSetGetVar( TO_bufferInfo * buf,
			    int             func,
			    TO_structInfo * info )
{
    TO_variableData * res = (TO_variableData *)initStructInfo( info, func );

    if( buf == (TO_bufferInfo *)0 || res == (TO_variableData *)0 )
	return DEC_BAD_PARM;

    if( doDecodeStr( buf, &res->variable ) != DEC_SUCCESS ||
	doDecodeStr( buf, &res->value    ) != DEC_SUCCESS )
    {
	TO_freeStructInfo( info );
	return buf->status;
    }

    return buf->status = DEC_SUCCESS;
}

/*----------------------------------------------------------------------------
|  Decoding
+---------------------------------------------------------------------------*/

int decCmdCreDa( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeDaemon( buf, CMD_CREATE_DAEMON, info );
}

int decCmdUpdDa( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeDaemon( buf, CMD_UPDATE_DAEMON, info );
}

int decCmdCmdDa( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeDaemon( buf, CMD_COMMAND_DAEMON, info );
}

int decCmdRefDa( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeDaemon( buf, CMD_REFRESH_DAEMON, info );
}

int decCmdAddAc( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeActor( buf, CMD_ADD_ACTOR, info );
}

int decCmdDelAc( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeActor( buf, CMD_DEL_ACTOR, info );
}

int decCmdAddPo( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodePort( buf, CMD_ADD_PORT, info );
}

int decCmdDelPo( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodePort( buf, CMD_DEL_PORT, info );
}

int decCmdAddSt( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeState( buf, CMD_ADD_STATE, info );
}

int decCmdDelSt( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeState( buf, CMD_DEL_STATE, info );
}

int decCmdAddCh( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeChain( buf, CMD_ADD_CHAIN, info );
}

int decCmdDelCh( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeChain( buf, CMD_DEL_CHAIN, info );
}

int decCmdAddVa( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeAddDelVar( buf, CMD_ADD_VAR, info );
}

int decCmdDelVa( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeAddDelVar( buf, CMD_DEL_VAR, info );
}

int decCmdSetVa( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeSetGetVar( buf, CMD_SET_VAR, info );
}

int decCmdGetVa( TO_bufferInfo * buf, TO_structInfo * info )
{
    return decodeSetGetVar( buf, CMD_GET_VAR, info );
}

int decCmdInjec( TO_bufferInfo * buf, TO_structInfo * info )
{
    TO_injectMsg * res = (TO_injectMsg *)initStructInfo( info, CMD_INJECT_MSG );

    if( buf == (TO_bufferInfo *)0 || res == (TO_injectMsg *)0 )
	return DEC_BAD_PARM;

    char * priority = (char *)0;

    if( doDecodeInt( buf, &res->inwards ) != DEC_SUCCESS ||
	doDecodeStr( buf, &res->signal  ) != DEC_SUCCESS ||
	doDecodeStr( buf, &priority     ) != DEC_SUCCESS ||
	doDecodeStr( buf, &res->data    ) != DEC_SUCCESS )
    {
	freeStr( priority );
	TO_freeStructInfo( info );
	return buf->status;
    }

    res->prio = RTPriority::lookup( priority );

    freeStr( priority );

    return buf->status = DEC_SUCCESS;
}

/*----------------------------------------------------------------------------
|  Freeing
+---------------------------------------------------------------------------*/

void freeCmdDaemo( TO_structInfo * info )
{
    TO_daemonSett * ptr = (TO_daemonSett *)info->ptr;

    freeStr( ptr->settings );
}

void freeCmdActor( TO_structInfo * info )
{
    TO_actorInfo * ptr = (TO_actorInfo *)info->ptr;

    freeStr( ptr->actor );
}

void freeCmdPort( TO_structInfo * info )
{
    TO_portInfo * ptr = (TO_portInfo *)info->ptr;

    freeStr( ptr->actor );
    freeStr( ptr->port  );
}

void freeCmdState( TO_structInfo * info )
{
    TO_stateInfo * ptr = (TO_stateInfo *)info->ptr;

    freeStr( ptr->actor );
    freeStr( ptr->state );
}

void freeCmdChain( TO_structInfo * info )
{
    TO_chainInfo * ptr = (TO_chainInfo *)info->ptr;

    freeStr( ptr->actor );
    freeStr( ptr->state );
    freeStr( ptr->chain );
}

void freeCmdAdDeV( TO_structInfo * info )
{
    TO_variableInfo * ptr = (TO_variableInfo *)info->ptr;

    freeStr( ptr->actor    );
    freeStr( ptr->variable );
}

void freeCmdSeGeV( TO_structInfo * info )
{
    TO_variableData * ptr = (TO_variableData *)info->ptr;

    freeStr( ptr->variable );
    freeStr( ptr->value    );
}

void freeCmdInjec( TO_structInfo * info )
{
    TO_injectMsg * ptr = (TO_injectMsg *)info->ptr;

    freeStr( ptr->signal );
    freeStr( ptr->data   );
}

/*||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||
||  Public functions
||
+|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/

int TO_encodeResultCode( TO_bufferInfo * buf,
			 TO_daemonInfo * daemon,
			 TO_resultCode * code )
{
    if( buf == (TO_bufferInfo *)0 || code == (TO_resultCode *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, RES_CODE, daemon ) != ENC_SUCCESS ||
	doEncodeInt       ( buf, code->code       ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtPresence( TO_bufferInfo  * buf,
			  TO_daemonInfo  * daemon,
			  TO_evtPresence * evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtPresence *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_PRESENCE, daemon ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->time           ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->actor          ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->actorID        ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->classID        ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtReference( TO_bufferInfo   * buf,
			   TO_daemonInfo   * daemon,
			   TO_evtReference * evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtReference *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_REFERENCE, daemon ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->time            ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->actorRef        ) != ENC_SUCCESS ||
	doEncodeInt       ( buf,  evt->incarns         ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtChain( TO_bufferInfo * buf,
		       TO_daemonInfo * daemon,
		       TO_evtChain   * evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtChain *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_CHAIN, daemon ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->time        ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->actor       ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->state       ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->chain       ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtVariable( TO_bufferInfo  *buf,
			  TO_daemonInfo  *daemon,
			  TO_evtVariable *evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtVariable *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_VARIABLE, daemon ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->time           ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->actor          ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->variable       ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->value          ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtMsgOut( TO_bufferInfo * buf,
			TO_daemonInfo * daemon,
			TO_evtMsgOut  * evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtMsgOut *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_MSG_OUT, daemon ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->sendTime      ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->sendActor     ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->sendPort      ) != ENC_SUCCESS ||
	doEncodeInt       ( buf,  evt->sendMethod    ) != ENC_SUCCESS ||
	doEncodeMessage   ( buf, &evt->message       ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtMsgIn( TO_bufferInfo * buf,
		       TO_daemonInfo * daemon,
		       TO_evtMsgIn   * evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtMsgIn *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_MSG_IN, daemon ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->recTime      ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->recActor     ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->recPort      ) != ENC_SUCCESS ||
	doEncodeMessage   ( buf, &evt->message      ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtMsgOutIn( TO_bufferInfo  * buf,
			  TO_daemonInfo  * daemon,
			  TO_evtMsgOutIn * evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtMsgOutIn *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_MSG_OUTIN, daemon ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->sendTime        ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->sendActor       ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->sendPort        ) != ENC_SUCCESS ||
	doEncodeInt       ( buf,  evt->sendMethod      ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->recTime         ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->recActor        ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->recPort         ) != ENC_SUCCESS ||
	doEncodeMessage   ( buf, &evt->message         ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtRtsStatus( TO_bufferInfo   * buf,
			   TO_daemonInfo   * daemon,
			   TO_evtRtsStatus * evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtRtsStatus *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_RTS_STATUS, daemon ) != ENC_SUCCESS ||
	doEncodeTimeStamp ( buf, &evt->time             ) != ENC_SUCCESS ||
	doEncodeInt       ( buf,  evt->status           ) != ENC_SUCCESS ||
	doEncodeInt       ( buf,  evt->byYou            ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

int TO_encodeEvtHistory( TO_bufferInfo * buf,
			 TO_daemonInfo * daemon,
			 TO_evtHistory * evt )
{
    if( buf == (TO_bufferInfo *)0 || evt == (TO_evtHistory *)0 )
	return ENC_BAD_PARM;

    if( doEncodeFuncDaemon( buf, EVT_HISTORY, daemon ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->actor         ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->state         ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->history       ) != ENC_SUCCESS ||
	doEncodeStr       ( buf,  evt->current       ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return buf->status = ENC_SUCCESS;
}

} // extern "C"
