/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTActorProbe_h__
#define __RTActorProbe_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTEventInfo_h_
#include <RTEventInfo.h>
#endif

#ifndef __RTProbe_h__
#include <RTProbe.h>
#endif

class  RTController;
class  RTDaemonInfo;
class  RTMessage;
struct RTObject_class;

class RTActorProbe : public RTProbe
{
public:
			    RTActorProbe( RTDebugger * debugger,
					  int          id,
					  const char * strPath,
					  int          actorIndex );
    virtual                 ~RTActorProbe( void );

    void                    evtHistory   ( int state );
    virtual void            evtPresence  ( int actorIndex,
					   int observerId,
					   int daemonId );
    virtual void            evtReference ( int compIndex,  int observerId );
    void                    evtMessageIn   ( const RTMessage * );
    void                    evtMessageOut  ( const RTMessage *,
					     RTEventInfo::SendMethod );
    void                    evtMessageOutIn( const RTMessage *,
					     RTEventInfo::SendMethod );
    void                    evtChain( int chainPos );
    void                    setChain( int               origin,
				      const char      * transition,
				      const RTMessage * );

    virtual int             connectProbe   ( void );
    virtual int             disconnectProbe( RTActor * actor );

    virtual const char    * getTypeStr( void ) const;
    int                     inject( int                    inwards,
				    const char           * signal,
				    int                    prio,
				    const void           * data,
				    const RTObject_class * type,
				    const RTDaemonInfo   & daemonInfo );

    void                    addFieldValue( const char * variable,
					    const char * value );
    void                    accessFieldValue( void );

private:
    struct FieldEntry
    {
		     FieldEntry( const char * aName, const char * aValue );
		     ~FieldEntry( void );

	char       * varName;
	char       * varValue;
	FieldEntry * next;
    }; //lint !e1712

    // Without this, some compilers disallow use of FieldEntry within itself!
    friend struct FieldEntry;

    int                     _chainOrigin;
    const char            * _chainTrans;
    const RTMessage       * _chainMsg;
    int                     _chainParent; // current state at pos 1
    int                     _chainState;  // current state at pos 2
    FieldEntry            * _fieldList;

    RTController          * context( void );
    RTActor               * getActorFromStrPath( void );

    // unavailable methods
    RTActorProbe( const RTActorProbe & );
    RTActorProbe & operator=( const RTActorProbe & );
}; //lint !e1712

#endif // __RTActorProbe_h__
