/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __endeTarg_h__
#define __endeTarg_h__ included

#ifndef __endeBoth_h__
#include <endeBoth.h>
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/*||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
||
||  Public Toolset functions:
||
+|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||*/

/*----------------------------------------------------------------------------
|  Encode information into the buf.
+-----------------------------------------------------------------------------
|  Common parameters:
|  - TO_bufferInfo *buf:
|    +  YOU malloc buf->data and set buf->size accordingly.
|  - TO_daemonInfo *daemon:
|    +  YOU set daemon->number to the desired daemon number.
|    +  YOU set daemon->invokeID to any string identifier.
+-----------------------------------------------------------------------------
|  If encoding was successful:
|  - returns ENC_SUCCESS and sets buf->status to same as return value
|  - fills buf->data with encoded bytes
|  - sets buf->used to number of bytes actually used
|
|  If encoding was unsuccessful:
|  - returns != ENC_SUCCESS (see enum encodingResult for details),
|      and sets buf->status to same as return value
|  - sets buf->used to roughly where in buf->data something went wrong
+-----------------------------------------------------------------------------
|  Example adding an actor to an existing daemon:
|
|  TO_bufferInfo buf;
|  TO_daemonInfo daemon;
|  TO_actorInfo  info;
|
|  // YOU set the buf.data, buf.size and daemon.* fields
|  // You don't need to set buf.used or buf.status
|
|  // YOU set all the info.* fields
|
|  TO_encodeAddActor( &buf, &daemon, &info );
|  switch( buf.status )
|  {
|    case ENC_SUCCESS:
|      // Cool! Please feel free to use buf.data in any way you like.
|      break;
|    case ENC_BAD_PARM:
|      // One or both parameters to TO_encodeAddActor were null pointers.
|      break;
|    case ENC_BAD_FUNC_TYPE:
|      // You tried to encode an unknown function number.
|      break;
|    case ENC_OVERFLOW:
|      // The buffer (buf.data) is too small. Try again with double size.
|      break;
|  }
+---------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
|  Result code from any Toolset operation
+---------------------------------------------------------------------------*/
int TO_encodeResultCode( TO_bufferInfo * buf,
			 TO_daemonInfo * daemon,
			 TO_resultCode * code );

/*----------------------------------------------------------------------------
|  Event Presence
+---------------------------------------------------------------------------*/
int TO_encodeEvtPresence( TO_bufferInfo  * buf,
			  TO_daemonInfo  * daemon,
			  TO_evtPresence * evt );

/*----------------------------------------------------------------------------
|  Event Reference
+---------------------------------------------------------------------------*/
int TO_encodeEvtReference( TO_bufferInfo   * buf,
			   TO_daemonInfo   * daemon,
			   TO_evtReference * evt );

/*----------------------------------------------------------------------------
|  Event Chain
+---------------------------------------------------------------------------*/
int TO_encodeEvtChain( TO_bufferInfo * buf,
		       TO_daemonInfo * daemon,
		       TO_evtChain   * evt );

/*----------------------------------------------------------------------------
|  Event Variable
+---------------------------------------------------------------------------*/
int TO_encodeEvtVariable( TO_bufferInfo  * buf,
			  TO_daemonInfo  * daemon,
			  TO_evtVariable * evt );

/*----------------------------------------------------------------------------
|  Event Message Out
+---------------------------------------------------------------------------*/
int TO_encodeEvtMsgOut( TO_bufferInfo * buf,
			TO_daemonInfo * daemon,
			TO_evtMsgOut  * evt );

/*----------------------------------------------------------------------------
|  Event Message In
+---------------------------------------------------------------------------*/
int TO_encodeEvtMsgIn( TO_bufferInfo * buf,
		       TO_daemonInfo * daemon,
		       TO_evtMsgIn   * evt );

/*----------------------------------------------------------------------------
|  Event Message OutIn
+---------------------------------------------------------------------------*/
int TO_encodeEvtMsgOutIn( TO_bufferInfo  * buf,
			  TO_daemonInfo  * daemon,
			  TO_evtMsgOutIn * evt );

/*----------------------------------------------------------------------------
|  Event Target Status
+---------------------------------------------------------------------------*/
int TO_encodeEvtRtsStatus( TO_bufferInfo   * buf,
			   TO_daemonInfo   * daemon,
			   TO_evtRtsStatus * evt );

/*----------------------------------------------------------------------------
|  Event History
+---------------------------------------------------------------------------*/
int TO_encodeEvtHistory( TO_bufferInfo * buf,
			 TO_daemonInfo * daemon,
			 TO_evtHistory * evt );

#ifdef __cplusplus
}
#endif

#endif // __endeTarg_h__
