/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2011. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTMain.h>

#if USE_THREADS && __GNUC__ < 3

// All this is here to protect memory allocation and dealocation calls that
// are not thread-safe in older versions of Cygwin.

#include <RTMutex.h>
#include <stdlib.h>

// Mutex to protect memory allocation and deallocation.
static RTMutex * memory_lock = (RTMutex *)0;

void * operator new( size_t size )
{
    void * p;

    if( memory_lock == (RTMutex *)0 )
	p = malloc( size );
    else
    {
	memory_lock->enter();
	p = malloc( size );
	memory_lock->leave();
    }

    return p;
}

void operator delete( void * p )
{
    if( p == (void *)0 )
    {
	// nothing to delete
    }
    else if( memory_lock == (RTMutex *)0 )
	free( p );
    else
    {
	memory_lock->enter();
	free( p );
	memory_lock->leave();
    }
}

void * operator new []( size_t size )
{
    return ::operator new( size );
}

void operator delete []( void * p )
{
    ::operator delete( p );
}

#endif // USE_THREADS && __GNUC__ < 3

int main( int argc, const char * const * argv )
{
#if USE_THREADS && __GNUC__ < 3
    memory_lock = new RTMutex;
#endif

    int rc = RTMain::entryPoint( argc, argv );

#if USE_THREADS && __GNUC__ < 3
    RTMutex * lock = memory_lock;

    memory_lock = (RTMutex *)0;

    delete lock;
#endif

    return rc;
}
