/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTSyncObject.h>
#include <RTTimespec.h>

void RTSyncObject::timedwait( const RTTimespec & expireTime )
{
    static int  initialized   = 0;
    static long nsec_per_tick;

    if( ! initialized )
    {
	// system_tick() returns number of microseconds per tick
	nsec_per_tick = system_tick() * 1000;
	initialized   = 1;
    }

    // convert timespec to ms
    OSTIME delta = (OSTIME)expireTime.ticksFromNow( nsec_per_tick );

    // only wait if delta is positive
    if( delta > 0 )
    {
	SIGSELECT   selector[] = { 1, RTOseSyncSignal };
	SIGNAL    * sig        = receive_w_tmo( delta, selector );

	// null sig value indicates a timeout
	if( sig != (SIGNAL *)0 )
	{
	    SIGSELECT number = sig->_number;

	    free_buf( &sig );

	    if( number != RTOseSyncSignal )
		RTDiag::panic( "RTSyncObject::timedwait( unexpected signal )" );
	}
    }
}
