/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTThreadInfo_h__
#define __RTThreadInfo_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTMutex_h__
#include <RTMutex.h>
#endif

#ifndef __RTSyncObject_h__
#include <RTSyncObject.h>
#endif

struct RTTask;

struct RTTaskCreate
{
    RTTaskCreate    * next;
    RTSyncObject      sync;
    RTTask          * task;
    void          ( * entry )( void * );
    void            * arg;
    unsigned long     priority;
    unsigned long     stack_size;
};

struct RTTaskJoin
{
    RTTaskJoin      * next;
    RTSyncObject      sync;
    RTTask          * task;
};

struct RTTask
{
    enum State
    {
	Running, Zombie, Reusable
    };

    RTTask        * next;
    RTTask        * prev;
    unsigned long   tid;
    unsigned long   stack_size;
    State           state;
    RTTaskJoin    * waiter;
};

struct RTThreadInfo
{
    RTTask * task;

    RTThreadInfo( void );
    ~RTThreadInfo( void );
};

class RTTaskManager
{
public:
			   RTTaskManager( void ( * entry )( void * ),
					  void   * arg,
					  int      priority,
					  int      stack_size );
			   ~RTTaskManager( void );

    static RTTask        * startTask( void ( * entry )( void * ),
				      void   * arg,
				      int      priority,
				      int      stack_size );

    static void            joinTask( RTTask * );

private:
    RTMutex                mutex;
    RTSyncObject           sync;

    RTTask               * tasks;
    int                    running;

    // requests from other threads
    RTTaskCreate         * creates;
    RTTaskJoin           * joins;

    RTTask               * assignTask( void          ( * entry )( void * ),
				       void            * arg,
				       unsigned long     priority,
				       unsigned long     stack_size );

    static void            wrapper( RTTask   *,
				    void   ( * entry )( void * ),
				    void     * arg );

    static RTTaskManager * soleInstance;
};

#endif // __RTThreadInfo_h__
