/*
 * Licensed Materials - Property of IBM
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.
 * U.S. Government Users Restricted Rights -  Use, duplication or
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTSyncObject.h>
#include <RTTimespec.h>

#include <errno.h>

void RTSyncObject::timedwait( const RTTimespec & expireTime )
{
    enter();

    if( signalled == 0 )
    {
	// the timeout argument to pthread_cond_timedwait is 'timespec *'
	// instead of 'RTTimespec *' so we must make a copy
	timespec timeout;

	timeout.tv_sec  = expireTime.tv_sec;
	timeout.tv_nsec = expireTime.tv_nsec;

	int rc = pthread_cond_timedwait( &cv, &mutex, &timeout );

	if( rc == 0 )
	{
	    // cv was signalled
	}
	else if( rc == ETIMEDOUT || errno == ETIMEDOUT )
	{
	    // timeout expired
	}
	else if( rc == EINTR || errno == EINTR )
	{
	    // interrupted by a signal
	}
	else
	{
	    RTDiag::panic( "pthread_cond_timedwait" );
	}
    }

    signalled = 0;

    leave();
}
