/*
 * Licensed Materials - Property of IBM
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS
 * Copyright IBM Corporation 1999, 2009. All Rights Reserved.
 * U.S. Government Users Restricted Rights -  Use, duplication or
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>

#include <stdio.h>
#include <sched.h>

int main( int argc, const char * const * argv )
{
    int policy   = -1;
    int priority = 0;

    for( int i = 0; ++i < argc; )
    {
	const char * option = argv[ i ];

	if( RTMemoryUtil::strcmp( option, "-SCHED_", 7 ) != 0 )
	    continue;

	option += 7;

	if(      RTMemoryUtil::strcmp( option, "FIFO",  4 ) == 0 )
	    option += 4, policy = SCHED_FIFO;
	else if( RTMemoryUtil::strcmp( option, "RR",    2 ) == 0 )
	    option += 2, policy = SCHED_RR;
	else if( RTMemoryUtil::strcmp( option, "OTHER", 5 ) == 0 )
	    option += 5, policy = SCHED_OTHER;
	else
	    continue;

	if( *option == '\0' )
	    priority = 0;
	else if( *option == '=' )
	    priority = RTMemoryUtil::atoi( &option[ 1 ] );
	else
	    continue;
    }

    while( policy != -1 )
    {
	int         priority_min;
	int         priority_max;
	sched_param sched;

	if( sched_getparam( 0, &sched ) != 0 )
	    perror( "sched_getparam" );

	else if( (priority_min = sched_get_priority_min( policy )) == -1 )
	    perror( "sched_get_priority_min" );

	else if( (priority_max = sched_get_priority_max( policy )) == -1 )
	    perror( "sched_get_priority_max" );

	else if( priority < priority_min || priority > priority_max )
	    RTDiag::out()
		<< "Priority (" << priority << ") out of range: ["
		<< priority_min << ".." << priority_max << "]\n";

	else if( sched.sched_priority = priority,
		 sched_setscheduler( 0, policy, &sched ) != 0 )
	    perror( "sched_setscheduler" );

	else
	    break;

	return 1;
    }

    return RTMain::entryPoint( argc, argv );
}
