/*
 * Licensed Materials - Property of IBM
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS
 * Copyright IBM Corporation 1999, 2009. All Rights Reserved.
 * U.S. Government Users Restricted Rights -  Use, duplication or
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTSyncObject.h>
#include <RTTimespec.h>

#include <errno.h>

void RTSyncObject::timedwait( const RTTimespec & expireTime )
{
    if( mutex_lock( &mutex ) != 0 )
	RTDiag::panic( "mutex_lock" );

    if( signalled == 0 )
    {
	// because the timeout argument to cond_timedwait is 'timespec *'
	// instead of 'RTTimespec *' we make a copy
	timespec timeout;

	timeout.tv_sec  = expireTime.tv_sec;
	timeout.tv_nsec = expireTime.tv_nsec;

	int rc = cond_timedwait( &cv, &mutex, &timeout );

	if( rc == 0 )
	{
	    // signalled
	}
	else if( rc == ETIME )
	{
	    // timeout
	}
	else if( rc == EINTR )
	{
	    // interrupted
	}
	else if( rc == EAGAIN )
	{
	    // ??
	}
	else if( errno != EINTR  && // someone has signaled me
		 errno != ETIME  && // the timeout value has expired
		 errno != EAGAIN )  // ??
	{
	    RTDiag::panic( "cond_timedwait" );
	}
    }

    signalled = 0;

    if( mutex_unlock( &mutex ) != 0 )
	RTDiag::panic( "mutex_unlock" );
}
