/*
 * Licensed Materials - Property of IBM
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS
 * Copyright IBM Corporation 1999, 2009. All Rights Reserved.
 * U.S. Government Users Restricted Rights -  Use, duplication or
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

extern "C"
{

static void * thread_run( void * job )
{
    ((RTJob *)job)->mainLoop();

    return (void *)0;
}

} // extern "C"

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job ( job )
    , _info( new RTThreadInfo )
{
    if( stack_size < 0 )
	stack_size = 0;

    unsigned ustack = (unsigned)stack_size;

    // make sure the stack is not too small
    if( ustack < thr_min_stack() )
	ustack = thr_min_stack();

    if( thr_create( (void *)0, ustack,
		    thread_run, (void *)job,
		    (long)THR_BOUND, &_info->tid ) != 0 )
	RTDiag::panic( "thr_create" );
    else
	(void)thr_setprio( _info->tid, prio );
}

RTThreadInfo::RTThreadInfo( void )
    : tid( (thread_t)0 )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( tid == (thread_t)0 )
	return; // never successfully created

    if( thr_join( tid, (thread_t *)0, (void **)0 ) != 0 )
	RTDiag::panic( "thr_join" );
}
