#! perl -w

# Licensed Materials - Property of IBM                               
# Component IBM Rational ROSE_RT BUILD : TARGET_RTS                 
# Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
# U.S. Government Users Restricted Rights -  Use, duplication or     
# disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 


# This script performs two functions: It translates object file lists
# (identified by %file on the command line) to linker command files,
# and adds the constructor/destructor tables to the executable.
#
# Example usage:
# ===============
# perl link.pl ARCH=386 -o TopCapsule \
#   main.o %OLIST user_obj.o -LX -lx ...
#
# The general algorithm:
#   Translate each OLIST to OLIST.cmd
#   dld -r -o RTmunch.o main.o -T OLIST.cmd user_obj.o -LX -lx
#   nm386 RTmunch.o | munch > RTctor.c
#   dcc -o RTctor.o -c RTctor.c
#   dld -r -o TopCapsule RTctor.o RTmunch.o

# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

sub mySystem
{
    my( $OS, $ROSERT_HOST );

    $OS          = $ENV{'OS'}          || '?';
    $ROSERT_HOST = $ENV{'ROSERT_HOST'} || '?';

    if( $ROSERT_HOST eq 'win32' && $OS !~ m{^Windows_NT$}i )
    {
	return system( join( ' ', @_ ) );
    }
    else
    {
        return system( @_ );
    }
}

sub report_ges_error
{
    my( $ERROR ) = @_;

    print "description '$ERROR' severity 'error'\n";
    print "result 'failure'\n";

    exit 1;
}

$arch_pattern = 'ARCH=(pentium|68k|960|ppc|simpc|simso)';

sub show_usage
{
    &report_ges_error( "Usage: $0 $arch_pattern options ... " );
}

&show_usage() unless $#ARGV >= 1;

if( $ARGV[0] =~ m{^$arch_pattern$} )
{
    $arch = $1;
    shift @ARGV;
}
else
{
    &show_usage();
}

&report_ges_error( "WIND_BASE is not set" )
	unless( $WIND_BASE = $ENV{'WIND_BASE'} );

# commands
$cc_cmd  = "dplus";


# filenames
$output  = 'a.out';

$status   = 0;                              # assume all is well
$cmdfile  = 'ld000';                        # command filename pattern
@tmpfiles = (); # temporary files

# These are for target-specific flags.

@prelink_flags = ();
@compile_flags = ();
@final_link_flags = ();

if( $arch eq 'ppc' )
{
    push @prelink_flags, '-tPPC603FH:rtp';
    push @compile_flags, '-tPPC603FH:rtp', '-DCPU=PPC603';
    push @final_link_flags, '-tPPC603FH:rtp';
    $linkpath = '-L'.$WIND_BASE.'\\target\\usr\\lib\\ppc\\PPC32\\common/'

}
if( $arch eq 'pentium' )
{   
    push @prelink_flags, '-tX86LH:rtpsim';
    push @compile_flags, '-tX86LH:rtpsim', '-DCPU=SIMPENTIUM';
    push @final_link_flags, '-tX86LH:rtpsim';
    $linkpath = '-L'.$WIND_BASE.'\\target\\usr\\lib\\simpentium\\SIMPENTIUM\\common/'    
}

@command = ();
@script_intro = ();

# Translate all indirect object file lists (identified by %) into linker
# command files for use in the prelink step.

foreach $arg ( @ARGV )
{
    if( $arg eq '-o' )
    {
	$lasto = 1;
    }
    elsif( $lasto )
    {
	$output = $arg;
	$lasto  = 0;
    }
    elsif( $arg =~ m{^%(.*)$} )
    {
	$in = $1;
	if( ! open( IN, "<$in" ) )
	{
	    print STDERR "link.pl: Can't read '$in': $!\n";
	    $status = 1;
	}
	else
	{
	    $out = "$cmdfile.cmd";
	    ++$cmdfile;
	    if( ! open( OUT, ">$out" ) )
	    {
		print STDERR "link.pl: Can't create '$out': $!\n";
		$status = 1;
	    }
	    else
	    {
		while( <IN> )
		{
		    chomp;
		    print OUT "$_.o\n";
		}
		close( OUT );

		push @command, @script_intro, "-@"."$out";
		@script_intro = ();
		push @tmpfiles, $out;
	    }
	    close( IN );
	}
    }
    else
    {
	push @command, $arg;
    }
}


@mycommand = ($cc_cmd, @compile_flags, '-Xansi' ,'-Xforce-declarations', 
              '-DTOOL_FAMILY=diab', '-DTOOL=diab', '-o' , $output);
push @mycommand, $linkpath, '-lstlstd', @command,;
$status = &mySystem( @mycommand ) if $status == 0;


exit $status if $status < 256;
exit $status / 256;
